package com.jinhaiqimeng.sysManageMicroservice.exception;

import cn.hutool.core.util.StrUtil;
import com.jinhaiqimeng.sysManageMicroservice.result.ResponseResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.BindException;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.ValidationException;
import java.util.concurrent.TimeoutException;
import java.util.stream.Collectors;

@RestControllerAdvice
@Slf4j
public class MyControllerAdvice {
    /**
     * 拦截Exception类的异常
     *
     * @param e
     * @return
     */
    @ExceptionHandler(Exception.class)
    public Object exceptionHandler(Exception e) {
        log.error("Exception content ",e);
        return ResponseResult.failed("系统错误，请联系管理员！");
    }

    /**
     * 拦截TimeoutException类的异常
     *
     * @param e
     * @return
     */
    @ExceptionHandler(TimeoutException.class)
    public Object exceptionHandler(TimeoutException e) {
        log.error("Exception content ",e);
        return ResponseResult.failed("执行超时！");
    }
    /**
     * 拦截 CommonException 的异常
     *
     * @param ex
     * @return
     */
    @ExceptionHandler(CommonException.class)
    public Object exceptionHandler(CommonException ex) {
        return ResponseResult.failed(ex.getMsg());
    }

    @ExceptionHandler(value = {BindException.class, ValidationException.class, MethodArgumentNotValidException.class})
    public Object handleValidatedException(Exception e) {
        String errorMsg = StrUtil.EMPTY;
        if (e instanceof MethodArgumentNotValidException) {
            // BeanValidation exception
            MethodArgumentNotValidException ex = (MethodArgumentNotValidException) e;
            errorMsg = ex.getBindingResult().getAllErrors().stream()
                    .map(ObjectError::getDefaultMessage)
                    .collect(Collectors.joining("; "));
        } else if (e instanceof ConstraintViolationException) {
            // BeanValidation GET simple param
            ConstraintViolationException ex = (ConstraintViolationException) e;
            errorMsg = ex.getConstraintViolations().stream()
                    .map(ConstraintViolation::getMessage)
                    .collect(Collectors.joining("; "));
        } else if (e instanceof BindException) {
            // BeanValidation GET object param
            BindException ex = (BindException) e;
            errorMsg = ex.getAllErrors().stream()
                    .map(ObjectError::getDefaultMessage)
                    .collect(Collectors.joining("; "));
        }
        return ResponseResult.failed(errorMsg);
    }
}
