package com.jinhaiqimeng.sysManageMicroservice.result;

import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.NoArgsConstructor;

/**
 * HTTP请求响应结果类
 *
 * @author shengn
 */
@NoArgsConstructor
public class ResponseResult<T> {

    private long status;
    private String msg;
    private T data;

    protected ResponseResult(long code, String msg, T data) {
        this.status = code;
        this.msg = msg;
        this.data = data;
    }

    /**
     * 成功返回结果
     *
     */
    public static <T> ResponseResult<T> success() {
        return new ResponseResult<T>(ResponseCode.SUCCESS.getCode(), ResponseCode.SUCCESS.getMessage(), null);
    }

    /**
     * 成功返回结果
     *
     * @param data 获取的数据
     */
    public static <T> ResponseResult<T> success(T data) {
        return new ResponseResult<T>(ResponseCode.SUCCESS.getCode(), ResponseCode.SUCCESS.getMessage(), data);
    }

    /**
     * 成功返回结果
     *
     * @param data    获取的数据
     * @param message 提示信息
     */
    public static <T> ResponseResult<T> success(T data, String message) {
        return new ResponseResult<T>(ResponseCode.SUCCESS.getCode(), message, data);
    }

    /**
     * 失败返回结果
     *
     * @param message 提示信息
     */
    public static <T> ResponseResult<T> failed(String message) {
        return new ResponseResult<T>(ResponseCode.FAILED.getCode(), message, null);
    }

    public static <T> ResponseResult<T> failed(Long code, String message) {
        return new ResponseResult<>(code, message, null);
    }


    public long getStatus() {
        return status;
    }

    public void setStatus(long status) {
        this.status = status;
    }

    public String getMsg() {
        return msg;
    }

    public void setMsg(String msg) {
        this.msg = msg;
    }

    public T getData() {
        return data;
    }

    public void setData(T data) {
        this.data = data;
    }

    @JsonIgnore
    public boolean isSuccess() {
        return ResponseCode.SUCCESS.getCode() == this.status;
    }

    @JsonIgnore
    public boolean isFail() {
        return !this.isSuccess();
    }
}
